package pds.xml;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.io.IOException;
import java.util.ArrayList;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.Locator;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

/**
 * XMLProcess is a class that will parse and process XML file 
 * and output a translated version. After creating an instance
 * set the output writer with a call to setWriter(), define
 * content for specific tags with addTag() (see @link{XMLTagContent})
 * and process a file with process().
 *
 * @author      Todd King
 * @author      Planetary Data System
 * @version     1.0, 10/30/03
 * @since       1.0
 */

public class XMLProcess extends DefaultHandler {
	private MultiWriter	out 			= new MultiWriter();
	private ArrayList	mTagList		= new ArrayList();	// Array of XMLTagContent
	private ArrayList	mReserveList	= new ArrayList();	// Array of XMLTagContent
	private static final int TagInclude = 0;
	private	String	mURI = "";
	
	public XMLProcess() {
		// Add single tag HTML elements
		setTag("DITDOS", "");
		setTag("BR", "<BR>");
		setTag("HR", "<HR>");
		
		setReserve("INCLUDE", TagInclude);
	}
	
	public void setDocumentLocator(Locator locator) {}
	public void startDocument() throws SAXException {}
	public void endDocument() throws SAXException {}
	public void startPrefixMapping(String prefix, String uri) throws SAXException {}
	public void endPrefixMapping(String prefix) throws SAXException {}
	public void skippedEntity(String name) throws SAXException {}  
	public void processingInstruction(String target, String data) throws SAXException {}
	
	public void characters(char[] text, int start, int length) throws SAXException {
		for(int i = 0; i < length; i++) out.print(text[start+i]); 
	}
	
	public void ignorableWhitespace(char[] text, int start, int length)	throws SAXException {
		for(int i = 0; i < length; i++) out.print(text[+i]); 
	}
	
	public void startElement(String namespaceURI, String localName,	String rawName, Attributes atts) throws SAXException {
		XMLTagContent	tag;
		String	pathName = "";
		
		if((tag = findTag(rawName)) != null) {	// Replace tag with content
			out.println(tag.start);
		} else if((tag = findReserve(rawName)) != null) {	// Replace tag with content
			switch(tag.id) {
			case TagInclude:
				for(int i = 0; i < atts.getLength(); i++) {
					if(atts.getQName(i).compareToIgnoreCase("REF") == 0) {
						pathName = atts.getValue(i);
					}
				}
				try {
					if(pathName.length() == 0) return;	// Error
					// Fix-up path name to support paths relative to processed file
					File file = new File(mURI);
					if(pathName.charAt(0) != file.separatorChar) {	// Relative path
						pathName = file.getParent() + file.separatorChar + pathName;
					}
					
					// Open include file and stream to output
					InputStreamReader stream = new InputStreamReader(new FileInputStream(pathName));
					char[]	buffer = new char[1024];
					int		n;
					
					while((n = stream.read(buffer)) != -1) {
						out.write(buffer, n);
					}
					stream.close();
				} catch(Exception e) {
					// Should do something
				}
				break;
			}
		} else {	// Print out what came in
			out.print("<" + rawName);
			for(int i = 0; i < atts.getLength(); i++) {
				out.print(" " + atts.getQName(i) + "=\"" + atts.getValue(i) + "\"");
			}
			out.print(">");
		}
	}
	
	public void endElement(String namespaceURI, String localName, String rawName) throws SAXException {
		XMLTagContent	tag;
		
		if((tag = findTag(rawName)) != null) {	// Replace tag with content
			out.println(tag.end);
		} else if((tag = findReserve(rawName)) != null) {	// Replace tag with content
			// Nothing to do yet
		} else {	// Print out what came in
			out.print("</" + rawName + ">"); 
		}
	} 
	
	public XMLTagContent findTag(String name) {
		return findItem(mTagList, name);
	}
	
	public XMLTagContent findReserve(String name) {
		return findItem(mReserveList, name);
	}
	
	public XMLTagContent findItem(ArrayList list, String name) {
		XMLTagContent	tag;
		
		for(int i = 0; i < list.size(); i++) {
			tag = (XMLTagContent) list.get(i);
			if(tag.name.equalsIgnoreCase(name)) return tag;
		}
		
		return null;
	}
	
	public XMLTagContent setTag(String name) {
		return setTag(name, "", "");
	}
	
	public XMLTagContent setTag(String name, int value) {
		return setTag(name, "" + value, "");
	}
	
	public XMLTagContent setTag(String name, long value) {
		return setTag(name, "" + value, "");
	}
	
	public XMLTagContent setTag(String name, String start) {
		return setTag(name, start, "");
	}
	
	public XMLTagContent setTag(String name, String start, String end) {
		XMLTagContent	tag;
		tag = findTag(name);
		if(tag == null) {
		 	tag = new XMLTagContent(name);
			mTagList.add(tag);
		}
		tag.start = start;
		tag.end = end;
		
		return tag;
	}
	
	public XMLTagContent setReserve(String name, int id) {
		XMLTagContent	tag;
		tag = findReserve(name);
		if(tag == null) {
		 	tag = new XMLTagContent(name);
			mReserveList.add(tag);
		}
		tag.id = id;
		
		return tag;
	}
	
	public void setWriter(PrintWriter out) {
		this.out.setWriter(out);
	}
	
	public void setWriter(PrintStream out) {
		this.out.setStream(out);
	}
	
	public void message(String text) {
		out.println(text);
	}
	
	public void write(char[] buffer, int n) {
		out.write(buffer, n);
	}
	
	public void reset() {
		mTagList.clear();
		out.reset();
	}
	
	// start parsing... 
	public void process(String uri) {
		if(out == null) {
			System.out.println("XMLPrint not configured properly.");
			return;
		}
		
		mURI = uri;
		try {
			SAXParser parser = SAXParserFactory.newInstance().newSAXParser();
			parser.parse(uri, this);
		}	  catch (SAXParseException e) { // well-formedness error
			out.println(uri + " is not well formed.");
			out.println(e.getMessage()
			+ " at line " + e.getLineNumber() 
			+ ", column " + e.getColumnNumber());
		} catch (SAXException e) { // some other kind of error
			out.println(e.getMessage());
		} catch (IOException e) {
			out.println("Could not report on " + uri + " because of the IOException " + e);
		} catch (Exception e) { // some other kind of error
			out.println(e.toString());
		}
	}
	
	// Make this a callable class
    /** 
     * The entry point when calling the class from the command line.
     * This can be used for testing purposes. The usage is:<BR>
     *<BR>
     * java XMLProcess URL1 [URL2 ...]
     *<BR>
	 * 
     * @param args    one or more URLs for files to process.
     *
     * @since           1.0
     */
	public static void main(String[] args) {
		XMLProcess xml = new XMLProcess();
		XMLTagContent	tag;
		
		if (args.length == 0) {
			System.out.println("Usage: java XMLPrint URL1 [URL2 ...]"); 
		} 
		
		xml.setWriter(System.out);
		tag = xml.setTag("content", "This is content");
		
		// start parsing... 
		for (int i = 0; i < args.length; i++) {
			xml.process(args[i]);
		}
	}
}
