package pds.util;

import java.util.ArrayList;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;

/**
 * PPIAction is a class that contains a ruleset action definition.
 *
 * @author      Todd King
 * @author      Planetary Data System
 * @version     1.0, 04/21/03
 * @since       1.0
 */
public class PPIProcess {
	ArrayList	mOutput = new ArrayList();
	
	/**
	 * Run a command line.
	 *
	 * Output from the command (or error messages) are available with the method
	 * {@see getOutput()}.
	 *
	 * @param command the command line to run.
	 *
	 * @return the exit status of the command.
	 *
	 * @since 1.0
	 **/
	public int run(String command)
	{
		Runtime	runtime;
		String	buffer;
		String	message;
		boolean dontParseArg;
		String	hold;
		// ArrayList	output = new ArrayList();
		Process		process;
		ArrayList	argList;
		InputStream	input;
		InputStream	error;
		OutputStream	outputStream;
		BufferedReader	inputReader;
		BufferedReader	errorReader;
		PPIThreadedReader	errorThread;
		PPIThreadedReader	inputThread;
		int		exitValue = -1;	// Failed to run
		
		buffer = command;
		try {
			// Prepare a runtime environment
			runtime = Runtime.getRuntime();

			// Determine how we should run application
			dontParseArg = false;
			hold = System.getProperty("os.name");
			if(hold.length() >= 6) { dontParseArg = (hold.substring(0, 6).compareToIgnoreCase("WINDOW") == 0); }
			
			// Replace variables then parse
			if(dontParseArg) {	// Let OS parse it
				process = runtime.exec(buffer);
			} else {
				buffer = buffer.replaceAll("\\\\n", "\\\\\\\\n"); // Preserve escapes
				buffer = buffer.replaceAll("\\\\r", "\\\\\\\\r"); // Preserve escapes
				argList = argSplit(buffer, true);
				String[] argArray = new String[argList.size()];
				argArray = (String[]) argList.toArray(argArray);
				process = runtime.exec(argArray);
			}
			
			// Collect output from command
			outputStream = process.getOutputStream();
			outputStream.close();
			
			input = process.getInputStream();
			inputReader = new BufferedReader(new InputStreamReader(input));
			inputThread = new PPIThreadedReader(inputReader, mOutput);
			
			error = process.getErrorStream();
			errorReader = new BufferedReader(new InputStreamReader(error));
			errorThread = new PPIThreadedReader(errorReader, mOutput);
			
			//start the threads
			inputThread.start();
			errorThread.start();
			
			//this method causes the current thread to wait until
			//the process object (the running external app)
			//is terminated.
			process.waitFor();	// Wait 2 seconds
			
			//read anything still in buffers.
			//join() waits for the threads to die.
			inputThread.join();
			errorThread.join();
			
			/*
			while((buffer = inputReader.readLine()) != null && (message = errorReader.readLine()) != null) {
				if(buffer != null) {
					System.out.println("Read line.:" + buffer);
					mOutput.add(buffer);
				}
				if(message != null) {
					System.out.println("Error.:" + message);
					mOutput.add(message);
				}
			}
			process.waitFor();	// Let it finish cleaning up
			*/
			exitValue = process.exitValue();
		} catch(Exception e) {
			buffer = "Unable to run: " + command;
			mOutput.add(buffer);
			mOutput.add(e.getMessage());
		}
		
		return exitValue;
	}
	
	/**
	 * Retrieve the output from the last run.
	 **/
	public ArrayList getOutput()
	{
		return mOutput;
	}
	

	 /** 
     * Split a string into one or more substrings by parsing on
     * whitespace and treating "!" and "=" as individual arguments. 
     * Arguments in quotes are kept intack. Escaped characters are
     * converted preserved with the escape removed.
	 * 
     * @param buffer	the string to parse.
     * @param onlyWhitespace	if <code>true</code> split the string 
     *					using only whitespace as a delimiter. If <code>false</code>
     *					the string is parsed on whitespace and "!" and "=" 
     *					are considered as individual arguments.
     *
     * @return          <code>ArrayList</code> an array of arguments.
     *
     * @since           1.0
     */
	static public ArrayList argSplit(String buffer, boolean onlyWhitespace) {
		ArrayList	arg = new ArrayList();
		int			n;
		char		c;
		String		temp = "";
		boolean		inQuote = false;
		
		n = buffer.length();
		for(int i = 0; i < n; i++) {
			c = buffer.charAt(i);
			if(c == '\\') { // Convert escaped characters
			  i++; 
			  if(i < n) {temp += buffer.charAt(i);} 
		    } else {
				if(c == '"') { if(inQuote) inQuote = false; else inQuote = true; continue; }
				if(inQuote) {
					temp += c;
				} else {
					if(onlyWhitespace) {
						if(Character.isWhitespace(c)) {	// Argument seperator
							if(temp.length() > 0) { arg.add(temp); temp = ""; }
						} else {
							temp += c;
						}
					} else {	// Use all tokens 
						if(Character.isWhitespace(c)) {	// Argument seperator
							if(temp.length() > 0) { arg.add(temp); temp = ""; }
						} else if(c == '!') {	// Seperator
							if(temp.length() > 0) { arg.add(temp); }
							arg.add("!"); 
							temp = "";
						} else if(c == '=') {	// Seperator
							if(temp.length() > 0) { arg.add(temp); }
							arg.add("="); 
							temp = "";
						} else {	// Add to buffer
							temp += c;
						}
					}
				}
			}
		}
		if(temp.length() > 0) { arg.add(temp); temp = ""; }
		
		return arg;
	}
}
